@@ -152,7 +152,7 @@ bool XAP_UnixFontHandle::doesGlyphExist(UT_UCS4Char g)
 // rec.top = distance from the origin to the top of the glyph
 // rec.height = total height of the glyph
 //
-bool  XAP_UnixFontHandle::glyphBox(UT_UCS4Char g, UT_Rect & rec) const
+bool  XAP_UnixFontHandle::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG) 
 {
 	XftFaceLocker locker(m_font->getLayoutXftFont(GR_CharWidthsCache::CACHE_FONT_SIZE));
 	FT_Face pFace = locker.getFace();
@@ -397,6 +397,24 @@ XAP_UnixFont::style XAP_UnixFont::getStyle(void) const
 	return m_style;
 }
 
+PangoStyle XAP_UnixFont::getPangoStyle(void) const
+{
+	PangoStyle ret = PANGO_STYLE_NORMAL;
+	if (m_style == STYLE_ITALIC || m_style == STYLE_BOLD_ITALIC) {
+		ret = PANGO_STYLE_ITALIC;
+	}
+	return ret;
+}
+
+PangoWeight XAP_UnixFont::getPangoWeight(void) const
+{
+	PangoWeight ret = PANGO_WEIGHT_NORMAL;
+	if (m_style == STYLE_BOLD || m_style == STYLE_BOLD_ITALIC || m_style == STYLE_BOLD_OUTLINE) {
+		ret = PANGO_WEIGHT_BOLD;
+	}
+	return ret;
+}
+
 const char *XAP_UnixFont::getFontfile(void) const
 {
 	return m_fontfile;
@@ -771,12 +789,13 @@ XftFont *XAP_UnixFont::getLayoutXftFont(UT_uint32 pixelsize) const
 // returns different fonts depending on the zoom percentage
 XftFont *XAP_UnixFont::getDeviceXftFont(UT_uint32 pixelsize, UT_uint32 zoomPercentage) const
 {
-	XftFont *pXftFont = getFontFromCache(pixelsize*zoomPercentage/100, false, zoomPercentage);
+#define extra_font_size 1.5
+	XftFont *pXftFont = getFontFromCache(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)) , false, zoomPercentage);
 	if (pXftFont)
 		return pXftFont;
 
-	fetchXftFont(pixelsize*zoomPercentage/100);
-	return getFontFromCache(pixelsize*zoomPercentage/100, false, zoomPercentage);
+	fetchXftFont(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)));
+	return getFontFromCache(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)), false, zoomPercentage);
 }
 
 XftFont *XAP_UnixFont::getFontFromCache(UT_uint32 pixelsize, bool /*bIsLayout*/, UT_uint32 /*zoomPercentage*/) const
