@@ -84,6 +84,9 @@ fp_Page::~fp_Page()
 		m_pOwner = NULL;
 		pDSL->deleteOwnedPage(this);
 	}
+
+	delete m_pHeader;
+	delete m_pFooter;
 }
 
 /*!
@@ -2210,6 +2213,8 @@ fp_Page::buildHdrFtrContainer(fl_HdrFtrSectionLayout* pHFSL,
 	{
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		(*ppHF)->getHdrFtrSectionLayout()->deletePage(this);
+
+		UT_ASSERT_HARMLESS( !*ppHF );
 	}
 	xxx_UT_DEBUGMSG(("SEVIOR: Building header container. page = %x hdrftr = %x \n",this,pHFSL));
 
