@@ -44,20 +44,42 @@ EMBED_LIB_DIR=@executable_path/../Frameworks
 #  -bz2
 
 WPDLIBS="libwpd-0.8.dylib libwpd-stream-0.8.dylib"
-libwpd_count=0
+MATHLIBS="libmathview_frontend_libxml2.0.dylib libmathview.0.dylib"
+
 DYLIBS="libgsf-1.1.dylib libgthread-2.0.dylib libgobject-2.0.dylib libglib-2.0.dylib libxml2.2.dylib libintl.1.dylib libiconv.2.dylib libbz2.1.dylib"
 
 # add optional libwpd...
+
+wpdlibs_all_found="yes"
+wpdlibs_not_found=""
+
 for i in $WPDLIBS ; do
-    if [ -f $i ] ; then
-        libwpd_count = $$liwpd_count + 1
+    if [ ! -f $FINK_LIB_DIR/$i ] ; then
+	wpdlibs_all_found="no"
+	wpdlibs_not_found="$wpdlibs_not_found $i"
     fi
 done
 
-if [ $libwpd_count = "2" ] ; then
+if [ $wpdlibs_all_found = "yes" ] ; then
     DYLIBS="$DYLIBS $WPDLIBS"
 else
-    echo "Can't find $WPDLIBS. WordPerfect plugin not built."
+    echo "Can't find $wpdlibs_not_found. WordPerfect plugin not built."
+fi
+
+mathlibs_all_found="yes"
+mathlibs_not_found=""
+
+for i in $MATHLIBS ; do
+    if [ ! -f $FINK_LIB_DIR/$i ] ; then
+	mathlibs_all_found="no"
+	mathlibs_not_found="$mathlibs_not_found $i"
+    fi
+done
+
+if [ $mathlibs_all_found = "yes" ] ; then
+    DYLIBS="$DYLIBS $MATHLIBS"
+else
+    echo "Can't find $mathlibs_not_found. abimathview plugin not built."
 fi
 
 for i in $DYLIBS ; do
