@@ -334,8 +334,8 @@ abi_control_gui_class_init (GObjectClass *klass)
 }
 
 GSF_CLASS_FULL (AbiControlGUI, abi_control_gui,
-		abi_control_gui_class_init, abi_control_gui_init,
-		G_TYPE_OBJECT, 0,
+		NULL, NULL, abi_control_gui_class_init, NULL,
+		abi_control_gui_init, G_TYPE_OBJECT, 0,
 		GSF_INTERFACE (abi_go_plot_data_allocator_init, GOG_DATA_ALLOCATOR_TYPE);
 		/*GSF_INTERFACE (abi_cmd_context_init, GO_CMD_CONTEXT_TYPE)*/)
 
