@@ -103,23 +103,23 @@ static int compareEntities(const void * pSz, const void * pEnt)
   sz1 = reinterpret_cast<const char*>(pSz);
   pE = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt)));
   sz2 = (*pE)->szEntity;
-  // printf("Compare sz1 %s to sz2 %s \n",sz1,sz2); 
   return UT_strcmp(sz1,sz2);
 }
 
 
 static int sortEntities(const void * pEnt1, const void * pEnt2)
 {
-  const char * sz1 =NULL;
-  const char * sz2 = NULL;
-  AbiMathViewEntityMapItem ** pE1 =NULL;
-  AbiMathViewEntityMapItem ** pE2 =NULL;
-  pE1 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt1)));
-  pE2 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt2)));
-  sz1 = (*pE1)->szEntity;
-  sz2 = (*pE2)->szEntity;
-  //printf("Compare sz1 %s to sz2 %s \n",sz1,sz2); 
-  return UT_strcmp(sz1,sz2);
+	const char * sz1 = NULL;
+	const char * sz2 = NULL;
+	AbiMathViewEntityMapItem ** pE1 = NULL;
+	AbiMathViewEntityMapItem ** pE2 = NULL;
+
+	pE1 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt1)));
+	pE2 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt2)));
+	sz1 = (*pE1)->szEntity;
+	sz2 = (*pE2)->szEntity;
+
+	return UT_strcmp(sz1,sz2);
 }
 
 //
@@ -307,28 +307,29 @@ static void AbiMathView_addToMenus()
 static void
 AbiMathView_removeFromMenus ()
 {
-  // First we need to get a pointer to the application itself.
-  XAP_App *pApp = XAP_App::getApp();
+	// First we need to get a pointer to the application itself.
+	XAP_App *pApp = XAP_App::getApp();
 
-  // remove the edit method
-  EV_EditMethodContainer* pEMC = pApp->getEditMethodContainer() ;
-  EV_EditMethod * pEM = ev_EditMethod_lookup ( "AbiMathView_FileInsert" ) ;
-  pEMC->removeEditMethod ( pEM ) ;
-  DELETEP( pEM ) ;
+	// remove the edit method
+	EV_EditMethodContainer* pEMC = pApp->getEditMethodContainer() ;
+	EV_EditMethod * pEM = ev_EditMethod_lookup ( "AbiMathView_FileInsert" ) ;
+	pEMC->removeEditMethod ( pEM ) ;
+	DELETEP( pEM ) ;
 
-  // now remove crap from the menus
-  int frameCount = pApp->getFrameCount();
-  XAP_Menu_Factory * pFact = pApp->getMenuFactory();
+	// now remove crap from the menus
+	int frameCount = pApp->getFrameCount();
+	XAP_Menu_Factory * pFact = pApp->getMenuFactory();
 
-  pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelEquation);
-  pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelFileInsert);
-  pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelLatexInsert);
-  for(int i = 0;i < frameCount;++i)
-  {
-      // Get the current frame that we're iterating through.
-      XAP_Frame* pFrame = pApp->getFrame(i);
-      pFrame->rebuildMenus();
-  }
+	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelEquation);
+	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelFileInsert);
+	pFact->removeMenuItem("Main",NULL,AbiMathView_MenuLabelLatexInsert);
+
+	for (int i = 0; i < frameCount; ++i)
+	{
+		// Get the current frame that we're iterating through.
+		XAP_Frame* pFrame = pApp->getFrame(i);
+		pFrame->rebuildMenus();
+	}
 }
  
 
@@ -337,53 +338,52 @@ XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFrame, con
 	XAP_String_Id String_id;
 
 	switch (errorCode)
-	  {
-	  case -301:
-		String_id = AP_STRING_ID_MSG_IE_FileNotFound;
-		break;
+	{
+		case -301:
+			String_id = AP_STRING_ID_MSG_IE_FileNotFound;
+			break;
 
-	  case -302:
-		String_id = AP_STRING_ID_MSG_IE_NoMemory;
-		break;
+		case -302:
+			String_id = AP_STRING_ID_MSG_IE_NoMemory;
+			break;
 
-	  case -303:
-		String_id = AP_STRING_ID_MSG_IE_UnsupportedType;
-		//AP_STRING_ID_MSG_IE_UnknownType;
-		break;
+		case -303:
+			String_id = AP_STRING_ID_MSG_IE_UnsupportedType;
+			//AP_STRING_ID_MSG_IE_UnknownType;
+			break;
 
-	  case -304:
-		String_id = AP_STRING_ID_MSG_IE_BogusDocument;
-		break;
+		case -304:
+			String_id = AP_STRING_ID_MSG_IE_BogusDocument;
+			break;
 
-	  case -305:
-		String_id = AP_STRING_ID_MSG_IE_CouldNotOpen;
-		break;
+		case -305:
+			String_id = AP_STRING_ID_MSG_IE_CouldNotOpen;
+			break;
 
-	  case -306:
-		String_id = AP_STRING_ID_MSG_IE_CouldNotWrite;
-		break;
+		case -306:
+			String_id = AP_STRING_ID_MSG_IE_CouldNotWrite;
+			break;
 
-	  case -307:
-		String_id = AP_STRING_ID_MSG_IE_FakeType;
-		break;
+		case -307:
+			String_id = AP_STRING_ID_MSG_IE_FakeType;
+			break;
 
-	  case -311:
-		String_id = AP_STRING_ID_MSG_IE_UnsupportedType;
-		break;
+		case -311:
+			String_id = AP_STRING_ID_MSG_IE_UnsupportedType;
+			break;
 
-	  default:
-		String_id = AP_STRING_ID_MSG_ImportError;
-	  }
+		default:
+			String_id = AP_STRING_ID_MSG_ImportError;
+	}
 
 	return pFrame->showMessageBox(String_id,
-									XAP_Dialog_MessageBox::b_O,
-									XAP_Dialog_MessageBox::a_OK,
-									pNewFile);
+			XAP_Dialog_MessageBox::b_O,
+			XAP_Dialog_MessageBox::a_OK,
+			pNewFile);
 }
 
 static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
-					   char ** ppPathname,
-					   IEGraphicFileType * iegft)
+					   char ** ppPathname)
 {
 	// raise the file-open dialog for inserting a MathML equation.
 	// return a_OK or a_CANCEL depending on which button
@@ -392,8 +392,7 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 	// pathname the user entered -- ownership of this goes
 	// to the caller (so free it when you're done with it).
 
-	UT_DEBUGMSG(("s_AskForMathMLPathname: frame %p\n",
-				 pFrame));
+	UT_DEBUGMSG(("s_AskForMathMLPathname: frame %p\n", pFrame));
 
 	UT_return_val_if_fail (ppPathname, false);
 	*ppPathname = NULL;
@@ -410,23 +409,12 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 	pDialog->setCurrentPathname(NULL);
 	pDialog->setSuggestFilename(false);
 
-	// to fill the file types popup list, we need to convert AP-level
-	// ImpGraphic descriptions, suffixes, and types into strings.
-
-	UT_uint32 filterCount = IE_ImpGraphic::getImporterCount();
-
-	const char ** szDescList = static_cast<const char **>(UT_calloc(filterCount + 1, sizeof(char *)));
-	const char ** szSuffixList = static_cast<const char **>(UT_calloc(filterCount + 1, sizeof(char *)));
-	IEGraphicFileType * nTypeList = (IEGraphicFileType *)
-		 UT_calloc(filterCount + 1,	sizeof(IEGraphicFileType));
-	UT_uint32 k = 0;
-
-	while (IE_ImpGraphic::enumerateDlgLabels(k, &szDescList[k], &szSuffixList[k], &nTypeList[k]))
-		k++;
+	/* 
+	TODO: add a "MathML (.xml)" entry to the file type list, and set is as the default file type
 
 	pDialog->setFileTypeList(szDescList, szSuffixList, static_cast<const UT_sint32 *>(nTypeList));
-	if (iegft != NULL)
-	  pDialog->setDefaultFileType(*iegft);
+	*/
+
 	pDialog->runModal(pFrame);
 
 	XAP_Dialog_FileOpenSaveAs::tAnswer ans = pDialog->getAnswer();
@@ -446,24 +434,23 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 		// suffixes to identify type (like Windows) will always
 		// want auto-detection.
 		if (type < 0)
+		{
 			switch (type)
 			{
 			case XAP_DIALOG_FILEOPENSAVEAS_FILE_TYPE_AUTO:
 				// do some automagical detecting
-				*iegft = IEGFT_Unknown;
 				break;
 			default:
 				// it returned a type we don't know how to handle
 				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			}
+		}
 		else
-			*iegft = static_cast<IEGraphicFileType>(pDialog->getFileType());
+		{
+			/* todo */
+		}
 	}
 
-	FREEP(szDescList);
-	FREEP(szSuffixList);
-	FREEP(nTypeList);
-
 	pDialogFactory->releaseDialog(pDialog);
 
 	return bOK;
@@ -477,58 +464,52 @@ static bool s_AskForMathMLPathname(XAP_Frame * pFrame,
 bool 
 AbiMathView_FileInsert(AV_View* v, EV_EditMethodCallData *d)
 {
-    // Get the current view that the user is in.
-    XAP_Frame *pFrame = XAP_App::getApp()->getLastFocussedFrame();
-    FV_View* pView = static_cast<FV_View*>(pFrame->getCurrentView());
-    PD_Document * pDoc = static_cast<PD_Document *>(pFrame->getCurrentDoc());
-    char* pNewFile = NULL;
+	// Get the current view that the user is in.
+	XAP_Frame *pFrame = XAP_App::getApp()->getLastFocussedFrame();
+	FV_View* pView = static_cast<FV_View*>(pFrame->getCurrentView());
+	PD_Document * pDoc = static_cast<PD_Document *>(pFrame->getCurrentDoc());
+	char* pNewFile = NULL;
 
 
-    IEGraphicFileType iegft = IEGFT_Unknown;
-    bool bOK = s_AskForMathMLPathname(pFrame,&pNewFile,&iegft);
+	bool bOK = s_AskForMathMLPathname(pFrame,&pNewFile);
 	
-    if (!bOK || !pNewFile)
-    {
-      UT_DEBUGMSG(("ARRG! bOK = %d pNewFile = %x \n",bOK,pNewFile));
-      return false;
-    }
-    UT_UTF8String sNewFile = pNewFile;
+	if (!bOK || !pNewFile)
+	{
+		UT_DEBUGMSG(("ARRG! bOK = %d pNewFile = %x \n",bOK,pNewFile));
+		return false;
+	}
+	UT_UTF8String sNewFile = pNewFile;
 
-    // we own storage for pNewFile and must free it.
-    FREEP(pNewFile);
+	// we own storage for pNewFile and must free it.
+	FREEP(pNewFile);
 
-
-    UT_DEBUGMSG(("fileInsertMathML: loading [%s]\n",sNewFile.utf8_str()));
+	UT_DEBUGMSG(("fileInsertMathML: loading [%s]\n",sNewFile.utf8_str()));
    
-    IE_Imp_MathML * pImpMathML = new IE_Imp_MathML(pDoc);
-    UT_Error errorCode = pImpMathML->importFile(sNewFile.utf8_str());
+	IE_Imp_MathML * pImpMathML = new IE_Imp_MathML(pDoc);
+	UT_Error errorCode = pImpMathML->importFile(sNewFile.utf8_str());
 
-    if(errorCode != UT_OK)
-    {
-      s_CouldNotLoadFileMessage(pFrame, sNewFile.utf8_str(), errorCode);
-      DELETEP(pImpMathML);
-      return false;
-    }
+	if (errorCode != UT_OK)
+	{
+		s_CouldNotLoadFileMessage(pFrame, sNewFile.utf8_str(), errorCode);
+		DELETEP(pImpMathML);
+		return false;
+	}
 
+	/* Create the data item */
+	const char* mimetypeMATHML = NULL;
+	mimetypeMATHML = UT_strdup("application/mathml+xml");
+	UT_uint32 uid = pDoc->getUID(UT_UniqueId::Image);
+	UT_UTF8String sUID;
+	UT_UTF8String_sprintf(sUID,"%d",uid);
+	pDoc->createDataItem(sUID.utf8_str(), false, pImpMathML->getByteBuf(), static_cast<void *>(const_cast<char *>(mimetypeMATHML)), NULL);
 
-    /*
-      Create the data item
-    */
-    const char* mimetypeMATHML = NULL;
-    mimetypeMATHML = UT_strdup("application/mathml+xml");
-    UT_uint32 uid = pDoc->getUID(UT_UniqueId::Image);
-    UT_UTF8String sUID;
-    UT_UTF8String_sprintf(sUID,"%d",uid);
-    pDoc->createDataItem(sUID.utf8_str(), false, pImpMathML->getByteBuf(), static_cast<void *>(const_cast<char *>(mimetypeMATHML)), NULL);
+	/* Insert the MathML Object */
+	PT_DocPosition pos = pView->getPoint();
+	pView->cmdInsertMathML(sUID.utf8_str(),pos);
 
-// Insert the MathML Object
-    PT_DocPosition pos = pView->getPoint();
-    //    printf("Insert Object at %d \n",pos);
-    pView->cmdInsertMathML(sUID.utf8_str(),pos);
+	DELETEP(pImpMathML);
 
-    DELETEP(pImpMathML);
-
-    return true;
+	return true;
 }
 
 
@@ -541,32 +522,32 @@ AbiMathView_FileInsert(AV_View* v, EV_EditMethodCallData *d)
 bool 
 AbiMathView_LatexInsert(AV_View* v, EV_EditMethodCallData *d)
 {
-  FV_View * pView = static_cast<FV_View *>(v);
-  XAP_Frame * pFrame = static_cast<XAP_Frame *> ( pView->getParentData());
-  pFrame->raise();
+	FV_View * pView = static_cast<FV_View *>(v);
+	XAP_Frame * pFrame = static_cast<XAP_Frame *> ( pView->getParentData());
+	pFrame->raise();
 
-  XAP_DialogFactory * pDialogFactory
-    = static_cast<XAP_DialogFactory *>(pFrame->getDialogFactory());
+	XAP_DialogFactory * pDialogFactory
+		= static_cast<XAP_DialogFactory *>(pFrame->getDialogFactory());
 
-  AP_Dialog_Latex * pDialog
+	AP_Dialog_Latex * pDialog 
 		= static_cast<AP_Dialog_Latex *>(pDialogFactory->requestDialog(AP_DIALOG_ID_LATEX));
-  UT_return_val_if_fail(pDialog, false);
-  if(pDialog->isRunning())
-    {
-      pDialog->activate();
-    }
-  else
-    {
-      pDialog->runModeless(pFrame);
-    }
-  bool bOK = true;
-  return bOK;
+	UT_return_val_if_fail(pDialog, false);
+
+	if (pDialog->isRunning())
+	{
+		pDialog->activate();
+	}
+	else
+	{
+		pDialog->runModeless(pFrame);
+	}
+
+	return true;
 }
 
-
 GR_AbiMathItems::GR_AbiMathItems(void): 
-  m_iAPI(0),
-  m_bHasSnapshot(false)
+	m_iAPI(0),
+	m_bHasSnapshot(false)
 {
 }
 
@@ -574,390 +555,465 @@ GR_AbiMathItems::~GR_AbiMathItems(void)
 {
 }
 
-
-GR_MathManager::GR_MathManager(GR_Graphics* pG)
-  : GR_EmbedManager(pG), 
-    m_CurrentUID(-1),
-    m_pLogger(NULL),
-    m_pMathGraphicDevice(NULL),
-    m_pAbiContext(NULL),
-    m_pOperatorDictionary(NULL),
-    m_pDoc(NULL)
+GR_MathManager::GR_MathManager(GR_Graphics* pG) :
+	GR_EmbedManager(pG), 
+	m_CurrentUID(-1),
+	m_pLogger(NULL),
+	m_pMathGraphicDevice(NULL),
+	m_pAbiContext(NULL),
+	m_pOperatorDictionary(NULL),
+	m_pDoc(NULL)
 {
-  m_vecMathView.clear();
-  m_vecItems.clear();
-  m_vecEntityMap;
+	m_vecMathView.clear();
+	m_vecItems.clear();
+	m_vecEntityMap;
 }
 
 GR_MathManager::~GR_MathManager()
 { 
-     DELETEP(m_pAbiContext);
-     m_pAbiContext = 0;
-     UT_VECTOR_PURGEALL(GR_AbiMathItems *,m_vecItems);
- }
+	DELETEP(m_pAbiContext);
+	m_pAbiContext = 0;
+	UT_VECTOR_PURGEALL(GR_AbiMathItems *,m_vecItems);
+}
 
 GR_EmbedManager * GR_MathManager::create(GR_Graphics * pG)
 {
-  return static_cast<GR_EmbedManager *>(new GR_MathManager(pG));
+	return static_cast<GR_EmbedManager *>(new GR_MathManager(pG));
 }
 
 const char * GR_MathManager::getObjectType(void) const
 {
-  return "mathml";
+	return "mathml";
 }
 
 void GR_MathManager::initialize(void)
 {
-     SmartPtr<AbstractLogger> logger = Logger::create();
-     m_pLogger = logger;
-     SmartPtr<Configuration> configuration = initConfiguration<libxml2_MathView>(logger, getenv("GTKMATHVIEWCONF"));
-     logger->setLogLevel(LOG_INFO);
-     SmartPtr<GR_Abi_MathGraphicDevice> mathGraphicDevice = GR_Abi_MathGraphicDevice::create(logger, configuration, getGraphics());
-     m_pMathGraphicDevice = mathGraphicDevice;
-     m_pAbiContext = new GR_Abi_RenderingContext(getGraphics());
-     m_pOperatorDictionary = initOperatorDictionary<libxml2_MathView>(logger, configuration);
+	XAP_App *pApp = XAP_App::getApp();
 
-     UT_sint32 i = 0;
-     AbiMathViewEntityMapItem * pEnt;
-     AbiMathViewEntityMapItem mi;
-     UT_sint32 MapSize = sizeof(mathmlEntitiesMap)/(sizeof(mi));
-     for(i=0; i<MapSize-1;i++)
-     {
-	 pEnt = const_cast<AbiMathViewEntityMapItem * >(&mathmlEntitiesMap[i]);
-	 m_vecEntityMap.addItem(pEnt);
-     }
-     m_vecEntityMap.qsort(sortEntities);
+	// load the gtkmathview settings and operator dictionaries from the private user directory, ...
+	UT_UTF8String userConfDir(pApp->getUserPrivateDirectory());
+	UT_UTF8String userDictDir(pApp->getUserPrivateDirectory());
+#if defined(WIN32)
+	userConfDir += "\\math\\gtkmathview.conf.xml";
+	userDictDir += "\\math\\dictionary.xml";
+#else
+	userConfDir += "/math/gtkmathview.conf.xml";
+	userDictDir += "/math/dictionary.xml";
+#endif
+
+	// ... or from the (common) AbiSuite directory
+	UT_UTF8String libConfDir(pApp->getAbiSuiteLibDir());
+	UT_UTF8String libDictDir(pApp->getAbiSuiteLibDir());
+#if defined(WIN32)	 
+	libConfDir += "\\math\\gtkmathview.conf.xml";
+	libDictDir += "\\math\\dictionary.xml";
+#else
+	libConfDir += "/math/gtkmathview.conf.xml";
+	libDictDir += "/math/dictionary.xml";
+#endif
+
+	// add the configuration paths
+	Configuration::addConfigurationPath( userConfDir.utf8_str() );
+	Configuration::addConfigurationPath( libConfDir.utf8_str() );
+ 
+	SmartPtr<AbstractLogger> logger = Logger::create();
+	m_pLogger = logger;
+	SmartPtr<Configuration> configuration = initConfiguration<libxml2_MathView>(logger, getenv("GTKMATHVIEWCONF"));
+	logger->setLogLevel(LOG_INFO);
+	 
+	// add the dictionary paths
+	configuration->add("dictionary/path", userDictDir.utf8_str());
+	configuration->add("dictionary/path", libDictDir.utf8_str());
+	 
+	SmartPtr<GR_Abi_MathGraphicDevice> mathGraphicDevice 
+			= GR_Abi_MathGraphicDevice::create(logger, configuration, getGraphics());
+	m_pMathGraphicDevice = mathGraphicDevice;
+	m_pAbiContext = new GR_Abi_RenderingContext(getGraphics());
+	m_pOperatorDictionary = initOperatorDictionary<libxml2_MathView>(logger, configuration);
+
+	UT_sint32 i = 0;
+	AbiMathViewEntityMapItem * pEnt;
+	AbiMathViewEntityMapItem mi;
+	UT_sint32 MapSize = sizeof(mathmlEntitiesMap)/(sizeof(mi));
+	for (i=0; i<MapSize-1;i++)
+	{
+		pEnt = const_cast<AbiMathViewEntityMapItem * >(&mathmlEntitiesMap[i]);
+		m_vecEntityMap.addItem(pEnt);
+	}
+	m_vecEntityMap.qsort(sortEntities);
 }
 
 UT_sint32  GR_MathManager::_makeMathView(void)
 {
-     SmartPtr<libxml2_MathView> pMathView = libxml2_MathView::create();
-     m_vecMathView.addItem(pMathView);
-     pMathView->setLogger(m_pLogger);
-     pMathView->setOperatorDictionary(m_pOperatorDictionary);
-     pMathView->setMathMLNamespaceContext(
+	UT_ASSERT(m_pLogger);
+	SmartPtr<libxml2_MathView> pMathView = libxml2_MathView::create(m_pLogger);
+	m_vecMathView.addItem(pMathView);
+	pMathView->setOperatorDictionary(m_pOperatorDictionary);
+	pMathView->setMathMLNamespaceContext(
 				     MathMLNamespaceContext::create(pMathView, 
                                      m_pMathGraphicDevice));
-     return static_cast<UT_sint32>(m_vecMathView.getItemCount()-1);
+	return static_cast<UT_sint32>(m_vecMathView.getItemCount()-1);
 }
 
 void GR_MathManager::_loadMathML(UT_sint32 uid, UT_UTF8String & sMathBuf)
 {
-  SmartPtr<libxml2_MathView> pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
-  pMathView->loadBuffer(sMathBuf.utf8_str());
+	SmartPtr<libxml2_MathView> pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
+	pMathView->loadBuffer(sMathBuf.utf8_str());
 }
 
 void GR_MathManager::setDefaultFontSize(UT_sint32 uid, UT_sint32 iSize)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
-  pMathView->setDefaultFontSize(iSize);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
+	pMathView->setDefaultFontSize(iSize);
 }
 
 UT_sint32 GR_MathManager::makeEmbedView(AD_Document * pDoc, UT_uint32 api, const char * szDataID)
 {
-  if(m_pDoc == NULL)
-  {
-    m_pDoc = static_cast<PD_Document *>(pDoc);
-  }
-  else
-  {
-    UT_ASSERT(m_pDoc == static_cast<PD_Document *>(pDoc));
-  }
-  UT_sint32 iNew = _makeMathView();
-  GR_AbiMathItems * pItem = new GR_AbiMathItems();
-  pItem->m_iAPI = api;
-  pItem->m_bHasSnapshot = false;
-  m_vecItems.addItem(pItem);
-  UT_ASSERT(static_cast<UT_sint32>(m_vecItems.getItemCount()) == (iNew+1));
-  return iNew;
+	if (m_pDoc == NULL)
+	{
+		m_pDoc = static_cast<PD_Document *>(pDoc);
+	}
+	else
+	{
+		UT_ASSERT(m_pDoc == static_cast<PD_Document *>(pDoc));
+	}
+
+	UT_sint32 iNew = _makeMathView();
+	GR_AbiMathItems * pItem = new GR_AbiMathItems();
+	pItem->m_iAPI = api;
+	pItem->m_bHasSnapshot = false;
+	m_vecItems.addItem(pItem);
+	UT_ASSERT(static_cast<UT_sint32>(m_vecItems.getItemCount()) == (iNew+1));
+
+	return iNew;
 }
 
 void GR_MathManager::makeSnapShot(UT_sint32 uid, UT_Rect & rec)
 {
-  if(!getGraphics()->queryProperties(GR_Graphics::DGP_SCREEN))
-    {
-      return;
-    }
-  GR_AbiMathItems * pItem = m_vecItems.getNthItem(uid);
-  UT_return_if_fail(pItem);  
-  const PP_AttrProp * pSpanAP = NULL;
-  PT_AttrPropIndex api = pItem->m_iAPI;
-  /* bool b = */ m_pDoc->getAttrProp(api, &pSpanAP);
-  const char * pszDataID = NULL;
-  bool bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
-  if(pItem->m_bHasSnapshot)
-    {
-       updatePNGSnapshot(static_cast<AD_Document *>(m_pDoc),rec,pszDataID);
-    }
-  else
-    {
-       createPNGSnapshot(static_cast<AD_Document *>(m_pDoc),rec,pszDataID);
-       pItem->m_bHasSnapshot = true;
-    }
+	const PP_AttrProp * pSpanAP = NULL;
+	const char * pszDataID = NULL;
+	bool bFoundDataID = false;
+
+	if (!getGraphics()->queryProperties(GR_Graphics::DGP_SCREEN))
+	{
+		return;
+	}
+
+	GR_AbiMathItems * pItem = m_vecItems.getNthItem(uid);
+	UT_return_if_fail(pItem);  
+	PT_AttrPropIndex api = pItem->m_iAPI;
+	m_pDoc->getAttrProp(api, &pSpanAP);
+
+	bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
+	UT_ASSERT((bFoundDataID));
+
+	if (pItem->m_bHasSnapshot)
+	{
+		updatePNGSnapshot(static_cast<AD_Document *>(m_pDoc),rec,pszDataID);
+	}
+	else
+	{
+		createPNGSnapshot(static_cast<AD_Document *>(m_pDoc),rec,pszDataID);
+		pItem->m_bHasSnapshot = true;
+	}
 }
 
 bool GR_MathManager::isDefault(void)
 {
-  return false;
+	return false;
 }
 
 
 bool GR_MathManager::createPNGSnapshot(AD_Document * pDoc, UT_Rect & rec,
 					   const char * szDataID)
 {
-  if(isDefault())
-  {
-    return false;
-  }
-  if((rec.width == 0) || (rec.height ==0))
-  {
-    return false;
-  }
-  GR_Painter painter(getGraphics());
-  GR_Image * pImage = painter.genImageFromRectangle(rec);
-  UT_ByteBuf * pBuf = NULL;
-  pImage->convertToBuffer(&pBuf);
-  UT_UTF8String sID = "snapshot-png-";
-  sID += szDataID;
-  const char* mimetypePNG = NULL;
-  mimetypePNG = UT_strdup("image/png");
-  printf("Making data-item of name %s \n",sID.utf8_str());
-  pDoc->createDataItem(sID.utf8_str(),false,reinterpret_cast< const UT_ByteBuf *>(pBuf),mimetypePNG,NULL);
-  delete pBuf;
-  delete pImage;
-  return true;
+	const char* mimetypePNG = NULL;
+
+	if (isDefault())
+	{
+		return false;
+	}
+
+	if ((rec.width == 0) || (rec.height ==0))
+	{
+		return false;
+	}
+
+	GR_Painter painter(getGraphics());
+	GR_Image * pImage = painter.genImageFromRectangle(rec);
+	UT_ByteBuf * pBuf = NULL;
+	pImage->convertToBuffer(&pBuf);
+	UT_UTF8String sID = "snapshot-png-";
+	sID += szDataID;
+
+	mimetypePNG = UT_strdup("image/png");
+	UT_DEBUGMSG(("Making data-item of name %s \n",sID.utf8_str()));
+	pDoc->createDataItem(sID.utf8_str(),false,reinterpret_cast< const UT_ByteBuf *>(pBuf),mimetypePNG,NULL);
+
+	DELETEP(pBuf);
+	DELETEP(pImage);
+
+	return true;
 }
 
 
 bool GR_MathManager::updatePNGSnapshot(AD_Document * pDoc, UT_Rect & rec,
 					   const char * szDataID)
 {
-  if(isDefault())
-  {
-    return false;
-  }
-  GR_Painter painter(getGraphics());
-  GR_Image * pImage = painter.genImageFromRectangle(rec);
-  UT_ByteBuf * pBuf = NULL;
-  pImage->convertToBuffer(&pBuf);
-  UT_UTF8String sID = "snapshot-png-";
-  sID += szDataID;
-  //printf("Updating data-item of name %s \n",sID.utf8_str());
-   pDoc->replaceDataItem(sID.utf8_str(),reinterpret_cast< const UT_ByteBuf *>(pBuf));
-  delete pBuf;
-  delete pImage;
-  return true;
+	if (isDefault())
+	{
+		return false;
+	}
+
+	GR_Painter painter(getGraphics());
+	GR_Image * pImage = painter.genImageFromRectangle(rec);
+	UT_ByteBuf * pBuf = NULL;
+
+	pImage->convertToBuffer(&pBuf);
+	UT_UTF8String sID = "snapshot-png-";
+	sID += szDataID;
+
+	pDoc->replaceDataItem(sID.utf8_str(),reinterpret_cast< const UT_ByteBuf *>(pBuf));
+
+	DELETEP(pBuf);
+	DELETEP(pImage);
+
+	return true;
 }
 
 bool GR_MathManager::modify(UT_sint32 uid)
 {
-  return false;
+	return false;
 }
 
 void GR_MathManager::initializeEmbedView(UT_sint32 uid)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
 }
 
 void GR_MathManager::loadEmbedData(UT_sint32 uid)
 {
-    SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
-  const PP_AttrProp * pSpanAP = NULL;
-  GR_AbiMathItems * pItem = m_vecItems.getNthItem(uid);
-  UT_return_if_fail(pItem);  
-  PT_AttrPropIndex api = pItem->m_iAPI;
-  /* bool b = */ m_pDoc->getAttrProp(api, &pSpanAP);
-  const char * pszDataID = NULL;
-  bool bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
-  printf("DataID val =%s \n",pszDataID);
-  UT_UTF8String sMathML;
-  if (bFoundDataID && pszDataID)
-  {
-       const UT_ByteBuf * pByteBuf = NULL;
-       bFoundDataID = m_pDoc->getDataItemDataByName(pszDataID, 
-						    const_cast<const UT_ByteBuf **>(&pByteBuf),
-						    NULL, NULL);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
 
-       UT_UCS4_mbtowc myWC;
-       sMathML.appendBuf( *pByteBuf, myWC);
-  }
-  UT_return_if_fail(bFoundDataID);
-  UT_return_if_fail(pszDataID);
-  UT_DEBUGMSG(("MATH ML string is... \n %s \n",sMathML.utf8_str()));
-  _loadMathML(uid, sMathML);
+	const PP_AttrProp * pSpanAP = NULL;
+	GR_AbiMathItems * pItem = m_vecItems.getNthItem(uid);
+	UT_return_if_fail(pItem);  
+
+	PT_AttrPropIndex api = pItem->m_iAPI;
+	m_pDoc->getAttrProp(api, &pSpanAP);
+
+	const char * pszDataID = NULL;
+	bool bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
+	UT_DEBUGMSG(("DataID val =%s \n",pszDataID));
+
+	UT_UTF8String sMathML;
+	if (bFoundDataID && pszDataID)
+	{
+		const UT_ByteBuf * pByteBuf = NULL;
+		bFoundDataID = m_pDoc->getDataItemDataByName(pszDataID, 
+						const_cast<const UT_ByteBuf **>(&pByteBuf),
+						NULL, NULL);
+
+		UT_UCS4_mbtowc myWC;
+		sMathML.appendBuf( *pByteBuf, myWC);
+	}
+
+	UT_return_if_fail(bFoundDataID);
+	UT_return_if_fail(pszDataID);
+	UT_DEBUGMSG(("MATH ML string is... \n %s \n",sMathML.utf8_str()));
+
+	_loadMathML(uid, sMathML);
 }
 
 UT_sint32 GR_MathManager::getWidth(UT_sint32 uid)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  BoundingBox box = pMathView->getBoundingBox();
-  return m_pAbiContext->toAbiLayoutUnits(box.width);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	BoundingBox box = pMathView->getBoundingBox();
+	return m_pAbiContext->toAbiLayoutUnits(box.width);
 }
 
-
 UT_sint32 GR_MathManager::getAscent(UT_sint32 uid)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  BoundingBox box = pMathView->getBoundingBox();
-  return m_pAbiContext->toAbiLayoutUnits(box.height);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	BoundingBox box = pMathView->getBoundingBox();
+	return m_pAbiContext->toAbiLayoutUnits(box.height);
 }
 
-
 UT_sint32 GR_MathManager::getDescent(UT_sint32 uid)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  BoundingBox box = pMathView->getBoundingBox();
-  return m_pAbiContext->toAbiLayoutUnits(box.depth);
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	BoundingBox box = pMathView->getBoundingBox();
+	return m_pAbiContext->toAbiLayoutUnits(box.depth);
 }
 
 void GR_MathManager::setColor(UT_sint32 uid, UT_RGBColor c)
 {
-  m_pAbiContext->setColor(c);
+	m_pAbiContext->setColor(c);
 }
 
 void GR_MathManager::render(UT_sint32 uid, UT_Rect & rec)
 {
-  scaled x = m_pAbiContext->fromAbiX(rec.left);
-  scaled y = m_pAbiContext->fromAbiLayoutUnits(-rec.top);// should be fromAbiY()
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
-  pMathView->render(*m_pAbiContext, x, y);
+	scaled x = m_pAbiContext->fromAbiX(rec.left);
+	scaled y = m_pAbiContext->fromAbiLayoutUnits(-rec.top);// should be fromAbiY()
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
+	pMathView->render(*m_pAbiContext, x, y);
 }
 
 void GR_MathManager::releaseEmbedView(UT_sint32 uid)
 {
-  SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
-  UT_return_if_fail(pMathView);
-  pMathView->resetRootElement();
+	SmartPtr<libxml2_MathView>  pMathView = m_vecMathView.getNthItem(uid);
+	UT_return_if_fail(pMathView);
+	pMathView->resetRootElement();
+}
+
+extern "C"
+{
+	extern void yyrestart(FILE *input_file);
+	extern int yyparse(char **str);
 }
 
 bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf & To)
 {
-  if(iConType != 0)
-  {
-    return false;
-  }
-  UT_UTF8String sLatex;
-  UT_UTF8String sMathML;
-  UT_UCS4_mbtowc myWC;
-  sLatex.appendBuf(From, myWC);
-  UT_UTF8String sCommand;
-  sCommand = "itex2MML <<FREDDDYYYY\n";
-  sCommand += sLatex;
-  sCommand += "\n";
-  sCommand += "FREDDDYYYY\n";
-  fflush(stdin);
-  FILE * psRes = popen(sCommand.utf8_str(),"r");
-  XAP_App * pApp = XAP_App::getApp();
-  XAP_Frame * pFrame = pApp->getLastFocussedFrame();
-  if(psRes == NULL)
-  {
-      pFrame->showMessageBox("itex2MML convertor not found on your system.\n Please install itex2MML \n",XAP_Dialog_MessageBox::b_O,
-			     XAP_Dialog_MessageBox::a_OK);
-    return false;
-  }
-  char FullLine[1003];  
-  UT_uint32 numRead = 1000;
-  while(numRead == 1000)
-  {
-    numRead = fread(&FullLine,1,1000,psRes);
-    FullLine[numRead] =0;
-    sMathML += FullLine;
-  }
-  //
-  // Fixme! Find out how to extract the error message.
-  //
-  pclose(psRes);
-  if(sMathML.size() == 0)
-  {
-      UT_UTF8String sErrMessage = "Error converting:\n";
-      sErrMessage += sLatex;
-      sErrMessage += "\n";
-      //      sErrMessage += FullLine;
-      pFrame->showMessageBox(sErrMessage.utf8_str(),
-				 XAP_Dialog_MessageBox::b_O,
-				 XAP_Dialog_MessageBox::a_OK);
-      return false;
-  }
-  char * szRawChars = NULL;
-  szRawChars = const_cast<char *>(sMathML.utf8_str());
-  printf("Input MathML %s \n",szRawChars);
-  char * psz0 = szRawChars;
-  char * psz1 = szRawChars;
-  char * psz2 = NULL;
-  UT_UTF8String sLeft;
-  bool bNew = false;
-  bool bLoop = true;
-  while(bLoop)
-  {
-    printf("Remaining string %s \n",psz0);
-       psz1 = strstr(psz0,"&");
-       if(psz1 == NULL)
-       {
-           bLoop = false;
-	   break;
-       }
-       psz2 = strstr(psz1,";");
-       if(psz2 == NULL)
-       {
-           bLoop = false;
-	   break;
-       }
-    //
-    // Make a copy of the string
-    //
-       UT_sint32 isize = static_cast<UT_sint32>(psz2-psz1);
-       char * pszNew = new char[isize];
-       pszNew[isize-1] = 0;
-       UT_sint32 i = 0;
-       for(i=0; i< (isize -1);i++)
-       {
-	    pszNew[i] = psz1[1+i];
-       }  
-       printf("Extracted entity %s \n",pszNew);
-        char * szEntVal = NULL;
-       UT_sint32 pos = m_vecEntityMap.binarysearch(reinterpret_cast<void *>(pszNew),compareEntities);
-       delete [] pszNew;
-       if(pos >= 0)
-       {
-	 //
-	 // OK slice up the string and but in the entity value
-	 //
-	    bNew = true;
- 	    AbiMathViewEntityMapItem * pE = m_vecEntityMap.getNthItem(pos);
-	    szEntVal = pE->szVal;
-	    *psz1 = 0; // rub out the '&'
-	    sLeft += psz0;
-	    sLeft += szEntVal;
-	    *psz2= 0;
-	    psz1 = psz2;
-	    psz1++;
-	    psz0 = psz1;
-       }
-       else
-       {
-	 UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	 bNew = false;
-	 break;
-       }
-       if(psz0 >= &szRawChars[sMathML.size()-1])
-       {
-	   bLoop = false;
-	   break;
-       }
-  }
-  if(bNew)
-  {
-       sLeft += psz0;
-       sMathML = sLeft;
-  }
-  printf("MATHML Inserted %s \n",sMathML.utf8_str());
-  To.ins(0,reinterpret_cast<const UT_Byte *>(sMathML.utf8_str()),static_cast<UT_uint32>(sMathML.size()));
-  return true;
+	XAP_App * pApp = XAP_App::getApp();
+	XAP_Frame * pFrame = pApp->getLastFocussedFrame();
+
+	if (iConType != 0)
+	{
+		return false;
+	}
+
+	/* add a pair of enclosing brackets \[ \] */
+	UT_UTF8String sLatex;
+	UT_UCS4_mbtowc myWC;
+	sLatex += "\\[";
+	sLatex.appendBuf(From, myWC);
+	sLatex += "\\]";
+
+	/* TODO: rewrite itex2MML to be able to use the From buffer directly
+		 instread of through a FILE* */
+
+	FILE *tmp = tmpfile();
+	fwrite(sLatex.utf8_str(), sLatex.size(), 1, tmp);
+	fflush(tmp);
+	rewind(tmp);
+	yyrestart(tmp);
+
+	char *mathml = 0;
+	int res = yyparse(&mathml);
+	fclose(tmp);
+	
+	if (res != 0)
+	{
+		pFrame->showMessageBox("TODO: give proper error message!\n",
+			XAP_Dialog_MessageBox::b_O, 
+			XAP_Dialog_MessageBox::a_OK);
+		return false;
+	}
+
+	UT_UTF8String sMathML(mathml);
+	FREEP(mathml);
+	
+	if (sMathML.size() == 0)
+	{
+		UT_UTF8String sErrMessage = "Error converting: <todo>\n";
+		//sErrMessage += sLatex;
+		sErrMessage += "\n";
+		// sErrMessage += FullLine;
+		pFrame->showMessageBox(sErrMessage.utf8_str(),
+				XAP_Dialog_MessageBox::b_O,
+				XAP_Dialog_MessageBox::a_OK);
+		return false;
+	}
+
+	char * szRawChars = NULL;
+	szRawChars = const_cast<char *>(sMathML.utf8_str());
+	UT_DEBUGMSG(("Input MathML %s \n", szRawChars));
+
+	char * psz0 = szRawChars;
+	char * psz1 = szRawChars;
+	char * psz2 = NULL;
+	UT_UTF8String sLeft;
+	bool bNew = false;
+	bool bLoop = true;
+	while (bLoop)
+	{
+		UT_DEBUGMSG(("Remaining string %s \n",psz0));
+
+		psz1 = strstr(psz0,"&");
+		if (psz1 == NULL)
+		{
+			bLoop = false;
+			break;
+		}
+
+		psz2 = strstr(psz1,";");
+		if(psz2 == NULL)
+		{
+			bLoop = false;
+			break;
+		}
+
+		//
+		// Make a copy of the string
+		//
+		UT_sint32 isize = static_cast<UT_sint32>(psz2-psz1);
+		char * pszNew = new char[isize];
+		pszNew[isize-1] = 0;
+		UT_sint32 i = 0;
+		for(i=0; i< (isize -1);i++)
+		{
+			pszNew[i] = psz1[1+i];
+		}
+		UT_DEBUGMSG(("Extracted entity %s \n", pszNew));
+
+		char * szEntVal = NULL;
+		UT_sint32 pos = m_vecEntityMap.binarysearch(reinterpret_cast<void *>(pszNew),compareEntities);
+		DELETEPV(pszNew);
+
+		if(pos >= 0)
+		{
+			//
+			// OK slice up the string and but in the entity value
+			//
+			bNew = true;
+			AbiMathViewEntityMapItem * pE = m_vecEntityMap.getNthItem(pos);
+			szEntVal = pE->szVal;
+			*psz1 = 0; // rub out the '&'
+			sLeft += psz0;
+			sLeft += szEntVal;
+			*psz2= 0;
+			psz1 = psz2;
+			psz1++;
+			psz0 = psz1;
+		}
+		else
+		{
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			bNew = false;
+			break;
+		}
+
+		if(psz0 >= &szRawChars[sMathML.size()-1])
+		{
+			bLoop = false;
+			break;
+		}
+	}
+
+	if (bNew)
+	{
+		sLeft += psz0;
+		sMathML = sLeft;
+	}
+	UT_DEBUGMSG(("MATHML Inserted %s \n",sMathML.utf8_str()));
+	To.ins(0,reinterpret_cast<const UT_Byte *>(sMathML.utf8_str()),static_cast<UT_uint32>(sMathML.size()));
+
+	return true;
 }
 
 static UT_uint32 MathManagerUID = 0; 
@@ -970,47 +1026,46 @@ ABI_PLUGIN_DECLARE(AbiMathView)
 //      Abiword Plugin Interface 
 //
 // -----------------------------------------------------------------------
-
   
 ABI_FAR_CALL
 int abi_plugin_register (XAP_ModuleInfo * mi)
 {
-    mi->name = "AbiMathView";
-    mi->desc = "The plugin allows AbiWord to be Grammar checked";
-    mi->version = ABI_VERSION_STRING;
-    mi->author = "Martin Sevior <msevior@physics.unimelb.edu.au>";
-    mi->usage = "No Usage";
+	mi->name = "AbiMathView";
+	mi->desc = "The plugin allows AbiWord to import MathML documents";
+	mi->version = ABI_VERSION_STRING;
+	mi->author = "Martin Sevior <msevior@physics.unimelb.edu.au>";
+	mi->usage = "No Usage";
     
-    // Add to AbiWord's plugin listeners
-    XAP_App * pApp = XAP_App::getApp();	
-    pMathManager = new GR_MathManager(NULL);
-    MathManagerUID = pApp->registerEmbeddable(pMathManager);
-   // Add to AbiWord's menus.
-    AbiMathView_addToMenus();
-     
-    return 1;
-}
+	// Add to AbiWord's plugin listeners
+	XAP_App * pApp = XAP_App::getApp();	
+	pMathManager = new GR_MathManager(NULL);
+	MathManagerUID = pApp->registerEmbeddable(pMathManager);
 
+	// Add to AbiWord's menus
+	AbiMathView_addToMenus();
+     
+	return 1;
+}
 
 ABI_FAR_CALL
 int abi_plugin_unregister (XAP_ModuleInfo * mi)
 {
-    mi->name = 0;
-    mi->desc = 0;
-    mi->version = 0;
-    mi->author = 0;
-    mi->usage = 0;
+	mi->name = 0;
+	mi->desc = 0;
+	mi->version = 0;
+	mi->author = 0;
+	mi->usage = 0;
 
-    XAP_App * pApp = XAP_App::getApp();
-    pApp->unRegisterEmbeddable(MathManagerUID);
-    DELETEP(pMathManager);
-    AbiMathView_removeFromMenus();
-    return 1;
+	XAP_App * pApp = XAP_App::getApp();
+	pApp->unRegisterEmbeddable(MathManagerUID);
+	DELETEP(pMathManager);
+	AbiMathView_removeFromMenus();
+
+	return 1;
 }
 
-
 ABI_FAR_CALL
 int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, UT_uint32 release)
 {
-    return 1; 
+	return 1; 
 }
