@@ -34,7 +34,7 @@ GR_Abi_CharArea::GR_Abi_CharArea(GR_Graphics* graphics, GR_Font* f, const scaled
   UT_ASSERT(f);
   UT_Rect glyphRect;
   graphics->setFont(m_pFont);
-  f->glyphBox(m_ch, glyphRect);
+  f->glyphBox(m_ch, glyphRect, graphics);
 #if 0
   fprintf(stderr, "getting glyph %d\n glyphBox [left=%d,width=%d,top=%d,height=%d]\n measureUnremappedChar=%d\n measureUnremappedCharForCache=%d\n ftlu=%f\n tdu=%f\n tlu=%f\n size=%d\n", c,
 	  glyphRect.left, glyphRect.width, glyphRect.top, glyphRect.height,
